from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from .base_boundary import *  # noqa: F401 E402 F403

__all__ = [name for name in dir() if not name.startswith('_')]



import compas
import compas_slicer

if __name__ == '__main__':
    print('COMPAS: {}'.format(compas.__version__))
    print('COMPAS Slicer: {}'.format(compas_slicer.__version__))
    print('Awesome! Your installation worked! :)')



import logging

logger = logging.getLogger('logger')

__all__ = ['zig_zag_open_paths']


def zig_zag_open_paths(slicer):
    """ Reverses half of the open paths of the slicer, so that they can be printed in a zig zag motion. """
    reverse = False
    for layer in slicer.layers:
        for i, path in enumerate(layer.paths):
            if not path.is_closed:
                if not reverse:
                    reverse = True
                else:
                    path.points.reverse()
                    reverse = False

                path.is_closed = True  # label as closed so that it is printed without interruption



__all__ = ['interpolation_slicing_default_param']


def interpolation_slicing_default_param(key):
    """ Returns the default parameters with the specified key. """
    if key in default_parameters:
        return default_parameters[key]
    else:
        raise ValueError('The parameter with key : ' + str(key) +
                         ' does not exist in the defaults of curved_slicing parameters. ')


default_parameters = \
    {
        # geodesics method
        'target_LOW_geodesics_method': 'exact_igl',
        'target_HIGH_geodesics_method': 'exact_igl',

        # union method for HIGH target
        # if all are false, then default 'min' method is used
        'target_HIGH_smooth_union': [False, [10.0]],  # blend radius
        'target_HIGH_chamfer_union': [False, [100.0]],  # size
        'target_HIGH_stairs_union': [False, [80.0, 3]],  # size, n-1 number of peaks

        'uneven_upper_targets_offset': 0,

    }



import logging
from compas_slicer.utilities import find_next_printpoint
import math
from compas.geometry import Vector, normalize_vector

logger = logging.getLogger('logger')

__all__ = ['set_wait_time_on_sharp_corners',
           'set_wait_time_based_on_extruder_toggle',
           'override_wait_time']


def set_wait_time_on_sharp_corners(print_organizer, threshold=0.5 * math.pi, wait_time=0.3):
    """
    Sets a wait time at the sharp corners of the path, based on the angle threshold.

    Parameters
    ----------
    print_organizer: :class:`compas_slicer.print_organization.BasePrintOrganizer`
    threshold: float
        angle_threshold
    wait_time: float
        Time in seconds to introduce to add as a wait time
    """
    number_of_wait_points = 0
    for printpoint, i, j, k in print_organizer.printpoints_indices_iterator():
        neighbors = print_organizer.get_printpoint_neighboring_items('layer_%d' % i, 'path_%d' % j, k)
        prev_ppt = neighbors[0]
        next_ppt = neighbors[1]

        if prev_ppt and next_ppt:
            v_to_prev = normalize_vector(Vector.from_start_end(printpoint.pt, prev_ppt.pt))
            v_to_next = normalize_vector(Vector.from_start_end(printpoint.pt, next_ppt.pt))
            a = abs(Vector(*v_to_prev).angle(v_to_next))

            if a < threshold:
                printpoint.wait_time = wait_time
                printpoint.blend_radius = 0.0  # 0.0 blend radius for points where the robot will wait
                number_of_wait_points += 1
    logger.info('Added wait times for %d points' % number_of_wait_points)


def set_wait_time_based_on_extruder_toggle(print_organizer, wait_type, wait_time=0.3):
    """
    Sets a wait time for the printpoints, either before extrusion starts,
    after extrusion finishes, or in both cases.

    Parameters
    ----------
    print_organizer: :class:`compas_slicer.print_organization.BasePrintOrganizer`
    wait_type: str
        wait_before_extrusion:  sets a wait time before extrusion (extruder_toggle False to True)
        wait_after_extrusion: sets a wait time after extrusion (extruder_toggle True to False)
        wait_before_and_after_extrusion: sets a wait time before, and after extrusion
        wait_at_sharp_corners: sets a wait time at the sharp corners of the path
    wait_time: float
        Time in seconds to introduce to add as a wait time
    """

    for printpoint in print_organizer.printpoints_iterator():
        assert printpoint.extruder_toggle is not None, \
            'You need to set the extruder toggles first, before you can automatically set the wait time'

    logger.info("Setting wait time")

    for printpoint, i, j, k in print_organizer.printpoints_indices_iterator():
        number_of_wait_points = 0
        next_ppt = find_next_printpoint(print_organizer.printpoints_dict, i, j, k)

        # for the brim layer don't add any wait times
        if not print_organizer.slicer.layers[i].is_brim and next_ppt:
            if wait_type == "wait_before_extrusion":
                if printpoint.extruder_toggle is False and next_ppt.extruder_toggle is True:
                    next_ppt.wait_time = wait_time
                    next_ppt.blend_radius = 0.0
                    number_of_wait_points += 1
            elif wait_type == "wait_after_extrusion":
                if printpoint.extruder_toggle is True and next_ppt.extruder_toggle is False:
                    next_ppt.wait_time = wait_time
                    next_ppt.blend_radius = 0.0
                    number_of_wait_points += 1
            elif wait_type == "wait_before_and_after_extrusion":
                if printpoint.extruder_toggle is False and next_ppt.extruder_toggle is True:
                    next_ppt.wait_time = wait_time
                    next_ppt.blend_radius = 0.0
                    number_of_wait_points += 1
                if printpoint.extruder_toggle is True and next_ppt.extruder_toggle is False:
                    next_ppt.wait_time = wait_time
                    next_ppt.blend_radius = 0.0
                    number_of_wait_points += 1
            else:
                logger.error('Unknown wait type : ' + str(wait_type))

        logger.info('Added wait times for %d points' % number_of_wait_points)


def override_wait_time(print_organizer, override_value):
    """
    Overrides the wait_time value for the printpoints with a user-defined value.

    Parameters
    ----------
    print_organizer: :class:`compas_slicer.print_organization.BasePrintOrganizer`
    override_value: float
        Value to override the wait_time values with.
    """
    for printpoint in print_organizer.printpoints_iterator():
        printpoint.wait_time = override_value


if __name__ == "__main__":
    pass



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import argparse
import glob
import os
import shutil

import compas
import compas.plugins
import compas_ghpython
import compas_rhino


@compas.plugins.plugin(category='install')
def installable_rhino_packages():
    return ['compas_slicer_ghpython']


@compas.plugins.plugin(category='install')
def after_rhino_install(installed_packages):
    if 'compas_slicer_ghpython' not in installed_packages:
        return []

    results = []

    try:
        version = _get_version_from_args()
        dstdir = _get_grasshopper_userobjects_path(version)
        srcdir = os.path.join(os.path.dirname(__file__), 'gh_components')
        userobjects = glob.glob(os.path.join(srcdir, '*.ghuser'))

        for src in userobjects:
            dst = os.path.join(dstdir, os.path.basename(src))
            shutil.copyfile(src, dst)

        results.append(('compas_slicer_ghpython', 'Installed {} GH User Objects on {}'.format(len(userobjects), dstdir), True))
    except PermissionError:
        raise Exception('Please close first all instances of Rhino and then rerun the command')

    return results


@compas.plugins.plugin(category='install')
def after_rhino_uninstall(installed_packages):
    if 'compas_slicer_ghpython' not in installed_packages:
        return []

    results = []

    try:
        version = _get_version_from_args()
        dstdir = _get_grasshopper_userobjects_path(version)
        srcdir = os.path.join(os.path.dirname(__file__), 'gh_components')
        userobjects = glob.glob(os.path.join(srcdir, '*.ghuser'))

        for src in userobjects:
            dst = os.path.join(dstdir, os.path.basename(src))
            os.remove(dst)

        results.append(('compas_slicer_ghpython', 'Uninstalled {} GH User Objects'.format(len(userobjects)), True))
    except PermissionError:
        raise Exception('Please close first all instances of Rhino and then rerun the command')

    return results


def _get_version_from_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--version', choices=['5.0', '6.0', '7.0'], default='6.0')
    args = parser.parse_args()
    return compas_rhino._check_rhino_version(args.version)


# TODO: Remove once this PR is released: https://github.com/compas-dev/compas/pull/802
# For now, we just fake it get_grasshopper_library_path()
def _get_grasshopper_userobjects_path(version):
    lib_path = compas_ghpython.get_grasshopper_library_path(version)
    userobjects_path = lib_path.split(os.path.sep)[:-1] + ['UserObjects']
    return os.path.sep.join(userobjects_path)
